/*--------------------------------------------------------------------------
 * Copyright (C) 2000 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included 
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
 * OTHER DEALINGS IN THE SOFTWARE.
 * 
 * Except as contained in this notice, the name of Dallas Semiconductor 
 * shall not be used except as stated in the Dallas Semiconductor 
 * Branding Policy. 
 *--------------------------------------------------------------------------
 *
 * alldev.C - This utility uses TMEX to dump all of the currently supported
 *            device types and descriptions.
 *
 * Compiler:  Microsoft Visual C 6.0
 * Version: 1.02
 * History: 1.00 -> 1.02  Added TMAdapterSpec call.  Corrected Field Index.
 *
 */

#define TMEXUTIL

#include "ibtmexcw.h"   

/* local prototypes */
void Exit_Prog(char *, short);

/* Global Variables to hold list information */
short PortType=1, PortNum=1;
uchar state_buffer[5120];
long session_handle;                         

/*----------------------------------------------------------------------
 *  This is the Main routine for READ01.
 */
void main(short argc, char **argv)
{
   short printmsg=1,showsyntax=FALSE,i,j,flag;
   char tstr[80];       
   Specification Spec;
   short ROM[9];

   /* check arguments to see if request instruction with '?' or too many */
   if (argc > 2)
      showsyntax = TRUE;
   if (argc >= 2)
   {
      if (argv[1][0] == '?' || argv[1][1] == '?')
         showsyntax = TRUE;
   }
   if (showsyntax)
   {
       printf("\nusage: alldev \n"
              "  - display all supported device types\n"
              "  - version 1.00\n");
       Exit_Prog("",BAD_ARGUMENT);
   }

   /* Read the default port */
   if (!TMReadDefaultPort(&PortNum,&PortType))
   {
      printf("ERROR, Could not read the default PortNum and PortType!\n");
      PortNum = 1;
      PortType = 1;    
   }

   /* print a header */  
   printf("\nAllDev Version 1.02\n"
         "Dallas Semiconductor Corporation\n"
         "Copyright (C) 1992-2000\n\n");
   Get_Version(tstr);
   printf("Main Driver: %s\n",tstr);

   /* loop to get a session handle and to a temperature conversion */
   for (;;)
   {
      /* attempt to get a session */
      session_handle = TMExtendedStartSession(PortNum,PortType,NULL);
      if (session_handle > 0)  
      {
         printf("\n(1)   (2)  (3)  (4)  (5)  (6)  (7)   (8)   (9)   (10)\n");
         printf("-------------------------------------------------------\n");

         for (i = 1; i <= 0x7F; i++)
         {
            /* set the family type */
            ROM[0] = i;
            TMRom(session_handle,state_buffer,ROM);   

            /* get specification */
            if (TMGetFamilySpec(session_handle,state_buffer,&Spec) == 1)
            {
               printf("%02X, ",i);
               for (j = 0; j < 6; j++)
                  printf("%4d,",Spec.features[j]);
               printf(" %s\n",Spec.dscrptn);
            }
      
         }
         printf("\nField Index:\n------------\n"
                "(1) Family code in hex\n"
                "(2) Number of regular memory pages\n"
                "(3) Length of regular memory page in bytes\n"
                "(4) Number of status memory pages\n"
                "(5) Length of status memory page in bytes\n"
                "(6) Max communication speed (0 regular, 1 Overdrive)\n"
                "(7) Memory type (see below)\n"
                "(8) iButton Part number\n"
                "(9) Part number in non-iButton package\n"
                "(10) Brief descriptions\n");

         printf("\nMemory Types:\n--------------\n"
                  "0 NOMEM - no user storage space or with\n"
                  "          non-standard structure.\n"
                  "1 NVRAM - non-volatile rewritable RAM.\n"
                  "2 EPROM1- EPROM (OTP).\n"
                  "          Contains an onboard 8-bit CRC data check.\n"
                  "3 EPROM2 - EPROM (OTP). TMEX Bitmap starting on status page 8\n"
                  "           Contains an onboard 16-bit CRC.\n" 
                  "4 EPROM3 - EPROM (OTP). TMEX Bitmap in upper nibble of byte 0 of status memory\n"
                  "           Contains an onboard 16-bit CRC data check.\n"
                  "5 EEPROM1 - EEPROM, one address byte\n"
                  "6 MNVRAM - non-volatile rewritable RAM with read-only non rolling-over page \n"
                  "           write cycle counters associated with last 1/4 of pages \n"
                  "           (3 minimum)\n"
                  "7 EEPROM2 - EEPROM. On board CRC16 for Write/Read memory. \n"
                  "            Copy Scratchpad returns an authentication byte (alternating 1/0). \n"
                  "8 NVRAM2 - non-volatile RAM. Contains an onboard 16-bit CRC.\n"
                  "9 NVRAM3 - non-volatile RAM with bit accessible memory.  Contains an onboard \n"
                  "           16-bit CRC. \n");

         /* Check out adapter specs */
         if (TMReadDefaultPort(&PortNum,&PortType))
         {
            if (TMGetTypeVersion(PortType,tstr) < 0)
               Exit_Prog("No Hardware Driver for this type found!",NO_DRIVERS);

            flag = TMSetup(session_handle);
            printf("\n\nPort Adapter:\n---------------------------------\n");
            printf("Default PortType : %d\n",PortType);
            printf("Default PortNum  : %d\n",PortNum);
            printf("Setup of the default port: %d\n",flag);
            printf("PortType version string: %s\n",tstr);
            if (flag == 1)
            {            
               if (TMGetAdapterSpec(session_handle,&Spec) == 1)
               {
                  printf("Adapter part number, alternative part number, description:\n  %s\n",
                          Spec.dscrptn);
                  printf("Features:\n   Overdrive (0-no/1-yes): %d\n",Spec.features[0]);
                  printf("   Strong pullup (0-no/1-yes): %d\n",Spec.features[1]);
                  printf("   EPROM programming (0-no/1-yes): %d\n",Spec.features[2]);
               }
            }
         }

         /* done */
         Exit_Prog("\n\nComplete\n",NORMAL_EXIT);   
      }
      else if (session_handle == 0)
      {                            
         /* check if need to print the waiting message */
         if (printmsg)
         {  
            printmsg = FALSE;
            printf("\nWaiting to get access to the MicroLan\n");
         }
      }
      else if (session_handle < 0)   
      {
         MessageBeep(MB_ICONEXCLAMATION);
         Exit_Prog("No Hardware Driver for this type found!",-NO_BASDRV_FOUND);
         break;
      }
   }
}

/*--------------------------------------------------------------------------
 *  Prints a message, beeps and exits the program.
 */
void Exit_Prog(char *msg, short errcd)
{
   /* print the message left justified */
   printf("\r%s",msg);

   /*  error beep if return code is not 0 */
   if (errcd != NORMAL_EXIT)
      MessageBeep(MB_ICONEXCLAMATION);
     
   /* end the session if one is open */
   TMEndSession(session_handle);

   /*  return error code passed */
   exit(errcd);
}

